SystemOrganization addCategory: #'GT-Playground'!

!SequenceableCollection methodsFor: '*GT-Playground' stamp: 'StefanReichhart 3/21/2015 09:52'!
takeFirst: anInteger
	^ self first: (self size min: anInteger)! !

!GLMSystemWindow methodsFor: '*GT-Playground' stamp: 'AndreiChis 9/8/2014 22:56'!
acceptAction: anAction 
	"Be compatible with edit:label:accept:.
	Not the best place for this method.
	Interim situation to move things forward..."
	self flag: #TODO.
	self model ifNotNil: [:aModel | aModel acceptAction: anAction].! !

!GLMSystemWindow methodsFor: '*GT-Playground' stamp: 'AndreiChis 9/8/2014 22:55'!
acceptContents: aString 
	"Be compatible with edit:label:accept:.
	Not the best place for this method.
	Interim situation to move things forward..."
	self flag: #TODO.
	self model ifNotNil: [:aModel | aModel acceptContents: aString].

	! !

!Duration methodsFor: '*GT-Playground' stamp: 'TudorGirba 3/16/2015 07:42'!
gtPlaygroundHumanReadableString
	self < 1 minute ifTrue: [ ^ 'just now' ].
	self < 2 minutes ifTrue: [ ^ '1 minute ago' ].
	self < 1 hour ifTrue: [ ^ self minutes asString , ' minutes ago' ].
	self < 1 day ifTrue: [ ^ self hours asString , ' hours ago' ].
	self < 1 year ifTrue: [ ^ self days asString , ' days ago' ].
	^ self printString! !

GLMCompositePresentation subclass: #GTPlayground
	instanceVariableNames: ''
	classVariableNames: 'GTPlaygroundEnabledStatus'
	poolDictionaries: ''
	category: 'GT-Playground'!

!GTPlayground commentStamp: 'TudorGirba 10/22/2014 07:37' prior: 0!
Playground is the correspondent of a classic Smalltalk Workspace. The problem with the name Workspace is that it implies that work should be carried out in this space, while this is not a best practice. Playground describes better the intention of providing a place in which we can quickly play with some code.

The model behind a Playground instance is a PlayPage object.

!!!!Use cases
The Playground can be used in two ways

# As a place to construct and try out code snippets.
# as an entry point into an inspection process.

In both cases, the ability of diving into objects to the right is an important feature.

!!!!Running
[ [ [ 
	self open.
 ] ] ]!

!GTPlayground class methodsFor: 'compatibility-tools' stamp: 'StefanReichhart 3/20/2015 22:04'!
edit: aString label: aLabel

"Open a new playground with the given contents"
"Compatibility with the original Workspace"
"#edit: aText label: labelString accept: anAction"
"	^(Smalltalk tools workspace openLabel: labelString)
		acceptContents:  aText;
		acceptAction: anAction;
		yourself."

"Open an editor on the given string/text"
	| page |
	"self halt"

	page := GTPlayPage new
		saveContent: aString;
		title: aLabel;
		yourself.
		
	self flag: #TODO. "We need to put an equivalent to the acceptAction thing."
	^ self new openOn: page! !

!GTPlayground class methodsFor: 'examples' stamp: 'AndreiChis 1/30/2015 12:28'!
examplePlaygroundWithBindings
	| playground wsBindings |
	wsBindings := Dictionary newFrom: { #number-> 1 }.
	playground := Smalltalk tools workspace
		openContents: 'number + 1'.
	playground
		setBindings: wsBindings.! !

!GTPlayground class methodsFor: 'settings' stamp: 'AndreiChis 8/29/2014 00:39'!
isGTPlaygroundEnabled

	^ GTPlaygroundEnabledStatus ifNil: [ GTPlaygroundEnabledStatus := Smalltalk tools workspace = self ]! !

!GTPlayground class methodsFor: 'instance creation' stamp: 'TudorGirba 10/18/2014 21:52'!
open
	"self open"
	| page |
	page := GTPlayPage new.
	^ self new openOn: page! !

!GTPlayground class methodsFor: 'instance creation' stamp: 'StefanReichhart 3/20/2015 22:04'!
openContents: aString
	"Open a new playground with the given contents. aString has to be Smalltalk code. The style is applied"
	

	| page |
	
	page := GTPlayPage new
		saveContent: aString;
		yourself.
	^ self new 
		openOn: page;
		yourself! !

!GTPlayground class methodsFor: 'instance creation' stamp: 'StefanReichhart 3/20/2015 22:04'!
openContents: aString label: aLabel
	"Open a new playground with the given contents"

	| page |

	page := GTPlayPage new
		saveContent: aString;
		title: aLabel;
		yourself.
	^ self new openOn: page! !

!GTPlayground class methodsFor: 'compatibility-tools' stamp: 'TudorGirba 10/18/2014 21:52'!
openLabel: aLabel
	| page |

	page := GTPlayPage new
		title: aLabel;
		yourself.
	^ self new openOn: page! !

!GTPlayground class methodsFor: 'tools registry' stamp: 'AndreiChis 8/28/2014 21:33'!
register

	self registerToolsOn: Smalltalk tools! !

!GTPlayground class methodsFor: 'tools registry' stamp: 'TudorGirba 2/9/2014 01:10'!
registerToolsOn: registry
	"Add ourselves to registry. See [Smalltalk tools]"
	"self registerToolsOn: Smalltalk tools" 
	registry register: self as: #workspace
! !

!GTPlayground class methodsFor: 'settings' stamp: 'AndreiChis 8/28/2014 21:33'!
setGTPlaygroundEnabledStatus: aBoolean
	| oldStatus |
	
	oldStatus := self isGTPlaygroundEnabled.
	GTPlaygroundEnabledStatus := aBoolean.
	oldStatus ~= GTPlaygroundEnabledStatus ifTrue: [ 
		GTPlaygroundEnabledStatus 
			ifTrue: [ self register ]
			ifFalse: [ Workspace registerToolsOn: Smalltalk tools ] ]
	
	! !

!GTPlayground class methodsFor: 'settings' stamp: 'TudorGirba 10/20/2014 16:39'!
settingsOn: aBuilder 
	<systemsettings>
	
	(aBuilder setting: #GTPlaygroundStatus)
		target: self;
		parent: #gt;
		getSelector: #isGTPlaygroundEnabled;
		setSelector: #setGTPlaygroundEnabledStatus:; 
		label: 'GTPlayground';
		description: 'Enable or disable the GTPlayground.'.
	(aBuilder setting: #cacheDirectory)
		parent: #gt;
		type: #Directory;
		target: GTPlayBook;
		description: 'The path to the local Playground cache that stores all Playground scripts in ph files' translated;
		label: 'Local Playground cache directory'
! !

!GTPlayground class methodsFor: 'tools registry' stamp: 'TudorGirba 2/11/2014 22:23'!
taskbarIcon
	"Answer the icon for the receiver in a task bar."

	^ Smalltalk ui icons iconNamed: #workspaceIcon! !

!GTPlayground class methodsFor: 'instance creation' stamp: 'TudorGirba 3/15/2015 21:49'!
title
	^ 'Playground'! !

!GTPlayground methodsFor: 'accessing' stamp: 'AndreiChis 9/8/2014 23:02'!
acceptAction: anAction
	"acceptAction := anAction."! !

!GTPlayground methodsFor: 'accessing' stamp: 'AndreiChis 9/30/2014 20:46'!
acceptContents: aString

	^ (self entity acceptContents: aString)
		ifTrue: [  
			self update.
			true]! !

!GTPlayground methodsFor: 'building' stamp: 'StefanReichhart 3/20/2015 22:05'!
actionsForPreviousContents
	^ GTPlayBook instance pages
		collect: [ :each | 
			GLMGenericAction new
				title: (each content asString contractTo: 50);
				action: [ :text | 
					text entity saveContent: each content.
					text update ];
				yourself ]! !

!GTPlayground methodsFor: 'building' stamp: 'StefanReichhart 3/21/2015 10:41'!
codeIn: a
	^ a pharoPlayground
		title: [ :page | 
			| label |
			label := GLMEditableLabelBrick new.
			label text: 'Untitled'.
			"below I define custom child-to-parent width dependencies to update tab's width
			when input is changed by the user. It is neccessary, because tab's width
			is set to fillSpace and it doesn't update automatically, as it doesn't depend
			on children, e.g. editable label. Editable label's width is set to fillSpace too,
			se we need to define two up-level dependencies"
			
			"tab's width should depend on label (maxWidth is dynamic)"
			label layout widthDependency: [ label owner ].
			"label's width depends on width of rubric edit field"
			label editBrick layout widthDependency: [ label ].
			label
				when: #onAccepted do: [ label text logCr ];
				"to make label take maximal only necessary space"
				maxWidth: [ label approximateWidth ];
				"textLogic: [ 'Modified ', (DateAndTime now - page modificationTime) gtPlaygroundHumanReadableString  ];"
				"top = 0, right = left = 5, bottom = 1"
				margin: #(0 5 1) ];
		format: [ :page | page content ];
		act: [ :text :page |
				text selectionInterval: (1 to: text text size).
				text highlightEvaluateAndDo: [ :result | text selection: result ] ]
			icon: GLMUIThemeExtraIcons glamorousGo
			on: $G
			entitled: 'Do it all and go';
		installGoAction;
		installDefaultSelectionActions;
		act: [ :text :page | page publish ]
			icon: GLMUIThemeExtraIcons glamorousSaveToUrl
			entitled: 'Remote publish';
		dynamicActions: [ self actionsForPreviousContents ];
		onChangeOfPort: #text act: [ :text :page | page saveContent: text text ]! !

!GTPlayground methodsFor: 'building' stamp: 'StefanReichhart 3/20/2015 22:06'!
compose
	self
		title: 'Playground';
		custom:
			(GTInspector new
				noTitle; 
				noActions;
				showFirst: [ :composite |
					self codeIn: composite ];
				yourself);
		on: GLMBrowserClosing do: [ 
			"This is a hack for the moment.
			We need a better mechanism from Glamour to help 
			us keep the model in sync"
			self entity
					saveContent: self first panes first presentations first text.
			GTPlayBook instance addPageIfInteresting: self entity ]! !

!GTPlayground methodsFor: 'accessing' stamp: 'AndreiChis 3/19/2015 17:49'!
currentBindings 

	^ self inspectorPresentation firstPresentation currentBindings! !

!GTPlayground methodsFor: 'accessing' stamp: 'AndreiChis 1/30/2015 11:35'!
inspectorPresentation

	^ self presentations first ! !

!GTPlayground methodsFor: 'accessing' stamp: 'AndreiChis 1/9/2015 11:17'!
label: aString
	"Set the window label to the given string"
	self title: aString.
	self update! !

!GTPlayground methodsFor: 'actions' stamp: 'AndreiChis 3/19/2015 17:52'!
resetBindings

	^ self inspectorPresentation firstPresentation resetBindings! !

!GTPlayground methodsFor: 'actions' stamp: 'AndreiChis 1/30/2015 11:42'!
setBindings: aDictionary

	self inspectorPresentation firstPresentation 
		variableBindings: [ aDictionary associations ];
		updateVariableBindings! !

!RxMatcher methodsFor: '*GT-Playground' stamp: 'StefanReichhart 3/21/2015 09:52'!
matchesIn: aString subexpressionsDo: aBlock
	self matchesOnStream: aString readStream subexpressionsDo: aBlock! !

!RxMatcher methodsFor: '*GT-Playground' stamp: 'StefanReichhart 3/21/2015 09:51'!
matchesOnStream: aStream subexpressionsDo: aBlock
	| subexpressions |
	[ self searchStream: aStream ] whileTrue: [ 
		subexpressions := ((1 to: self subexpressionCount) collect: [ :each | (self subexpression: each) ]).
		aBlock value: subexpressions first value: subexpressions allButFirst ]! !

Object subclass: #GTPlayBook
	instanceVariableNames: 'pages pagesLimit'
	classVariableNames: 'instance'
	poolDictionaries: ''
	category: 'GT-Playground'!

GTPlayBook class
	instanceVariableNames: 'cacheDirectory'!

!GTPlayBook commentStamp: 'TudorGirba 10/22/2014 07:51' prior: 0!
This class holds the pages that are loaded in the image.

At class initialization time, it loads pages from the cacheDirectory and then maintains the pages in memory.!

GTPlayBook class
	instanceVariableNames: 'cacheDirectory'!

!GTPlayBook class methodsFor: 'accessing' stamp: 'StefanReichhart 3/21/2015 09:33'!
cacheDirectory
	^ cacheDirectory ifNil: [ cacheDirectory := self defaultCacheDirectory ]! !

!GTPlayBook class methodsFor: 'accessing' stamp: 'TudorGirba 10/20/2014 16:40'!
cacheDirectory: aString 
	cacheDirectory := aString asFileReference! !

!GTPlayBook class methodsFor: 'protocol' stamp: 'TudorGirba 12/5/2014 00:05'!
cleanUp
	
	instance := nil! !

!GTPlayBook class methodsFor: 'accessing-defaults' stamp: 'AndreiChis 11/5/2014 19:14'!
defaultCacheDirectory
	^ 'play-cache/' asFileReference ensureCreateDirectory! !

!GTPlayBook class methodsFor: 'initializing' stamp: 'StefanReichhart 3/21/2015 09:29'!
initialize
	" we have to make sure the cached references are up-to-date after the image starts up "
	Smalltalk addToStartUpList: self! !

!GTPlayBook class methodsFor: 'accessing' stamp: 'TudorGirba 10/18/2014 21:48'!
instance
	^ instance ifNil: [ instance := self new ]! !

!GTPlayBook class methodsFor: 'protocol' stamp: 'TudorGirba 10/19/2014 21:34'!
reset
	
	instance := nil! !

!GTPlayBook class methodsFor: 'initializing' stamp: 'StefanReichhart 3/21/2015 09:14'!
startUp
	cacheDirectory := nil.! !

!GTPlayBook class methodsFor: 'initializing' stamp: 'StefanReichhart 3/21/2015 09:14'!
startUp: resuming
	cacheDirectory := nil.! !

!GTPlayBook methodsFor: 'actions' stamp: 'StefanReichhart 3/21/2015 09:37'!
addPage: aPage
	self pages addFirst: aPage! !

!GTPlayBook methodsFor: 'actions' stamp: 'StefanReichhart 3/20/2015 22:48'!
addPageIfInteresting: aPage
	aPage isInteresting ifFalse: [ ^ self ].
	self addPage: aPage! !

!GTPlayBook methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 3/20/2015 22:40'!
allFiles
	^ self class cacheDirectory files select: [ :each | 
		each extension = GTPlayPage fileExtension ]! !

!GTPlayBook methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 3/21/2015 09:38'!
allFilesSorted
	^ self allFiles sorted: [ :a :b | 
		a modificationTime > b modificationTime ]! !

!GTPlayBook methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 3/20/2015 22:45'!
defaultPages
	^ OrderedCollection new! !

!GTPlayBook methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 3/21/2015 09:47'!
defaultPagesLimit
	" page list in ui might get too long / holding too many object might decrease performance "
	^ 100! !

!GTPlayBook methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 3/21/2015 09:36'!
files
	^ self allFilesSorted takeFirst: self pagesLimit! !

!GTPlayBook methodsFor: 'ui' stamp: 'StefanReichhart 3/21/2015 09:39'!
gtPagesIn: composite
	<gtInspectorPresentationOrder: 0>
	composite list 
		title: 'Pages';
		display: [ self pages ];
		format: #gtDisplayString;
		filterOn: [ :text :each | text match: each content ]! !

!GTPlayBook methodsFor: 'initializing' stamp: 'StefanReichhart 3/21/2015 09:36'!
initialize
	self flag: 'error handling'.
	
	self pages: self defaultPages.
	self files do: [ :each | 
		[ self pages add: (GTPlayPage fromFileReference: each) ] " bypass "
			on: Error 
			do: [ " unhandled exceptions are bad design - we should handle it somehow " ] ]! !

!GTPlayBook methodsFor: 'accessing' stamp: 'StefanReichhart 3/20/2015 22:45'!
pages
	^ pages ifNil: [ pages := self defaultPages ]! !

!GTPlayBook methodsFor: 'accessing' stamp: 'TudorGirba 10/18/2014 21:49'!
pages: anObject
	pages := anObject! !

!GTPlayBook methodsFor: 'accessing' stamp: 'StefanReichhart 3/20/2015 22:34'!
pagesLimit
	^ pagesLimit ifNil: [ self defaultPagesLimit ]! !

!GTPlayBook methodsFor: 'accessing' stamp: 'StefanReichhart 3/20/2015 22:33'!
pagesLimit: anObject
	pagesLimit := anObject! !

!GTPlayBook methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 3/21/2015 09:38'!
pagesSorted
	^ self pages sorted: [ :a :b | 
		a modificationTime > b modificationTime ]! !

!GTPlayBook methodsFor: 'actions' stamp: 'StefanReichhart 3/21/2015 09:48'!
removePage: aPage
	self pages remove: aPage ifAbsent: [ " ignore if aPage is not registered " ]! !

Object subclass: #GTPlayPage
	instanceVariableNames: 'content creationTime fileName fileReference modificationTime publishUrl title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Playground'!

!GTPlayPage commentStamp: 'TudorGirba 10/22/2014 07:52' prior: 0!
It holds one page of code. It is the model behind the Playground.

It also knows how to store itself on disk, and every time its content changes, it updates the corresponding file.!

!GTPlayPage class methodsFor: 'instance creation' stamp: 'StefanReichhart 3/20/2015 23:43'!
creationTimeFrom: aFileReference
	^ [ self creationTimeFromString: aFileReference basenameWithoutExtension ] 
		on: Error 
		do: [ aFileReference creationTime ]! !

!GTPlayPage class methodsFor: 'instance creation' stamp: 'StefanReichhart 3/21/2015 09:50'!
creationTimeFromString: aString
	" this is really ugly, but hopefully only a temporary solution untill we come up with somthing more reliable and more beautiful "
	| subexpressions |
	subexpressions := OrderedCollection new.
	'(\d+)-(\d+)-(\d+)T(\d+)-(\d+)-(\d+)(-(\d+))?([-+]\d+)-(\d+)(-(\d+))?' asRegex 
		matchesIn: aString
		subexpressionsDo: [ :expression :expressionFragments | 
			subexpressions addAll: (expressionFragments collect: [ :each | 
				[ each squeezeOutNumber ] on: Error do: [ 0 ] ]) ].
	^ DateAndTime 
		year: subexpressions first 
		month: subexpressions second 
		day: subexpressions third 
		hour: subexpressions fourth 
		minute: subexpressions fifth 
		second: subexpressions sixth 
		nanoSecond: subexpressions eighth * 100000 " 7 == full match "
		offset: (Duration 
						days: 0 
						hours: subexpressions ninth 
						minutes: (subexpressions at: 10) 
						seconds: (subexpressions at: 12)) " 11 == full match "! !

!GTPlayPage class methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 3/20/2015 22:31'!
fileExtension
	^ 'ph'! !

!GTPlayPage class methodsFor: 'instance creation' stamp: 'StefanReichhart 3/21/2015 00:14'!
fromFileReference: aFileReference
	^ GTPlayPage new 
		fileName: aFileReference basename;
		content: (aFileReference readStreamDo: #contents); 
		creationTime: (self creationTimeFrom: aFileReference);
		modificationTime: aFileReference modificationTime;
		yourself! !

!GTPlayPage methodsFor: 'compatibility-tools' stamp: 'StefanReichhart 3/21/2015 09:45'!
acceptContents: aStringOrtext

	self flag: #TODO. "Need support for what lies in the comment down here"
	"self fileName ifNotNil: [ self saveString: aStringOrtext toFile: self fileName ].

	acceptAction ifNotNil:[acceptAction value: aStringOrtext]."
	self saveContent: aStringOrtext.
	^ true! !

!GTPlayPage methodsFor: 'compatibility-tools' stamp: 'StefanReichhart 3/20/2015 22:10'!
cacheDirectory
	self deprecated: 'use: #fileDirectory instead'.
	^ self fileDirectory! !

!GTPlayPage methodsFor: 'accessing' stamp: 'StefanReichhart 3/21/2015 09:41'!
content
	" lazy for better backward/forward-compatibility "
	^ content ifNil: [ content := self defaultContent ]! !

!GTPlayPage methodsFor: 'accessing' stamp: 'StefanReichhart 3/20/2015 22:06'!
content: aString
	content := aString! !

!GTPlayPage methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 3/21/2015 09:46'!
contentString
	" strange, sometimes #content is aText not aString !! "
	^ self content asString! !

!GTPlayPage methodsFor: 'accessing' stamp: 'StefanReichhart 3/21/2015 09:41'!
creationTime
	" lazy for better backward/forward-compatibility "
	^ creationTime ifNil: [ creationTime := self defaultCreationTime ]! !

!GTPlayPage methodsFor: 'accessing' stamp: 'StefanReichhart 3/20/2015 21:49'!
creationTime: anObject
	creationTime := anObject! !

!GTPlayPage methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 3/20/2015 21:52'!
defaultContent
	^ ''! !

!GTPlayPage methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 3/20/2015 21:54'!
defaultCreationTime
	^ self defaultTimestamp! !

!GTPlayPage methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 3/20/2015 23:00'!
defaultFileName
	^ self title , '.' , self fileExtension! !

!GTPlayPage methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 3/20/2015 21:54'!
defaultModificationTime
	^ self creationTime! !

!GTPlayPage methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 3/20/2015 21:51'!
defaultTimestamp
	^ DateAndTime now! !

!GTPlayPage methodsFor: 'private' stamp: 'StefanReichhart 3/20/2015 22:18'!
deleteFromFileSystem
	self fileReference ensureDelete! !

!GTPlayPage methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 3/20/2015 22:09'!
fileDirectory
	^ GTPlayBook cacheDirectory! !

!GTPlayPage methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 3/20/2015 22:31'!
fileExtension
	^ self class fileExtension! !

!GTPlayPage methodsFor: 'accessing' stamp: 'StefanReichhart 3/21/2015 09:41'!
fileName
	" lazy for better backward/forward-compatibility "
	^ fileName ifNil: [ fileName := self defaultFileName ]! !

!GTPlayPage methodsFor: 'accessing' stamp: 'StefanReichhart 3/20/2015 23:00'!
fileName: anObject
	fileName := anObject! !

!GTPlayPage methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 3/20/2015 22:09'!
fileReference
	^ self fileDirectory / self fileName! !

!GTPlayPage methodsFor: 'ui' stamp: 'StefanReichhart 3/21/2015 09:44'!
gtDisplayOn: stream
	stream nextPutAll: (self contentString copyReplaceAll: String cr with: String space)! !

!GTPlayPage methodsFor: 'ui' stamp: 'TudorGirba 11/2/2014 22:14'!
gtInspectorContentIn: composite
	<gtInspectorPresentationOrder: 0>
	composite pharoPlayground 
		title: 'Content';
		display: #content;
		installGoAction;
		installDefaultSelectionActions! !

!GTPlayPage methodsFor: 'initializing' stamp: 'StefanReichhart 3/21/2015 09:43'!
initialize
	super initialize.
	
	self content: self defaultContent.
	self creationTime: self defaultCreationTime.
	self modificationTime: self defaultModificationTime! !

!GTPlayPage methodsFor: 'testing' stamp: 'StefanReichhart 3/21/2015 09:44'!
isInteresting
	^ self contentString trimBoth isEmpty not! !

!GTPlayPage methodsFor: 'accessing' stamp: 'StefanReichhart 3/21/2015 09:41'!
modificationTime
	" lazy for better backward/forward-compatibility "
	^ modificationTime ifNil: [ modificationTime := self defaultModificationTime ]! !

!GTPlayPage methodsFor: 'accessing' stamp: 'TudorGirba 3/15/2015 23:24'!
modificationTime: anObject
	modificationTime := anObject! !

!GTPlayPage methodsFor: 'actions' stamp: 'StefanReichhart 3/21/2015 09:45'!
publish
	(self confirm: 'Publish playground content to the cloud?')
        ifFalse: [ ^ self ].
    self content ifEmpty: [ 
        self inform: 'Nothing was published because the playground is empty'.
        ^ self ].
    Clipboard clipboardText: (publishUrl := (GTUrlProvider new post: self contentString) asString).
    self inform: publishUrl , ' was published and the url was copied to clipboard'! !

!GTPlayPage methodsFor: 'accessing' stamp: 'StefanReichhart 3/20/2015 22:14'!
publishUrl
	^ publishUrl! !

!GTPlayPage methodsFor: 'accessing' stamp: 'StefanReichhart 3/20/2015 22:13'!
publishUrl: anObject
	publishUrl := anObject! !

!GTPlayPage methodsFor: 'actions' stamp: 'StefanReichhart 3/20/2015 22:17'!
save
	self content isEmpty
		ifTrue: [ self deleteFromFileSystem ]
		ifFalse: [ self writeToFileSystem ]! !

!GTPlayPage methodsFor: 'actions' stamp: 'StefanReichhart 3/21/2015 00:18'!
saveContent: aStringOrText
	self modificationTime: self defaultTimestamp.
	self content: aStringOrText.
	self save.! !

!GTPlayPage methodsFor: 'compatibility-tools' stamp: 'StefanReichhart 3/21/2015 09:45'!
silentContent: aStringOrtext
	self deprecated: 'use: #content: instead (has no side-effects)'.
	self content: aStringOrtext! !

!GTPlayPage methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 3/20/2015 22:10'!
title
	^ (self creationTime asString copyReplaceAll: ':' with: '-') copyReplaceAll: '.' with: '-'! !

!GTPlayPage methodsFor: 'private' stamp: 'StefanReichhart 3/21/2015 09:44'!
writeToFileSystem
	self fileReference writeStreamDo: [ :stream | 
		stream 
			truncate; 
			nextPutAll: self contentString ]! !

Object subclass: #GTUrlProvider
	instanceVariableNames: 'urlProvider'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Playground'!

GTUrlProvider class
	instanceVariableNames: 'urlProvider'!

!GTUrlProvider commentStamp: 'TudorGirba 10/22/2014 07:50' prior: 0!
This class was inspired from the Zinc SharedWorkspace project.

Its job is to work with a UrlProvider that can provide (via get) and store (via post) strings.!

GTUrlProvider class
	instanceVariableNames: 'urlProvider'!

!GTUrlProvider class methodsFor: 'settings' stamp: 'TudorGirba 10/22/2014 08:02'!
settingsOn: aBuilder 
	<systemsettings>
	
	(aBuilder setting: #stringUrlProvider)
		parent: #gt;
		target: self;
		description: 'The url for the script sharing service like the one from http://ws.stfx.eu' translated;
		label: 'The url for the script sharing service'
! !

!GTUrlProvider class methodsFor: 'accessing' stamp: 'TudorGirba 10/22/2014 08:02'!
stringUrlProvider
	^ self urlProvider asString! !

!GTUrlProvider class methodsFor: 'accessing' stamp: 'TudorGirba 10/22/2014 08:01'!
stringUrlProvider: aString
	self urlProvider: aString! !

!GTUrlProvider class methodsFor: 'accessing' stamp: 'TudorGirba 10/22/2014 07:48'!
urlProvider
	^ urlProvider ifNil: [ urlProvider := 'http://ws.stfx.eu' asZnUrl ]! !

!GTUrlProvider class methodsFor: 'accessing' stamp: 'TudorGirba 10/22/2014 07:48'!
urlProvider: aUrlObject
	urlProvider := aUrlObject ifNotNil: [ aUrlObject asZnUrl ]! !

!GTUrlProvider methodsFor: 'public' stamp: 'TudorGirba 10/22/2014 07:50'!
get: url	
	| contents |
	contents := self httpClient get: url.
	^ String new: contents size streamContents: [ :stream |
		contents linesDo: [ :each | stream nextPutAll: each; cr ] ]! !

!GTUrlProvider methodsFor: 'private' stamp: 'TudorGirba 10/22/2014 07:47'!
httpClient
	^ ZnClient new
		systemPolicy;
		beOneShot;
		accept: ZnMimeType textPlain;
		url: self urlProvider;
		yourself! !

!GTUrlProvider methodsFor: 'public' stamp: 'TudorGirba 10/22/2014 07:53'!
post: string
	| entity client url |
	entity := ZnEntity text: string asString.
	(client := self httpClient)
		entity: entity; 
		post.
	url := client response location.
	^ url asZnUrl inContextOf: self urlProvider ! !

!GTUrlProvider methodsFor: 'public' stamp: 'TudorGirba 10/22/2014 07:49'!
urlProvider
	^ urlProvider ifNil: [ urlProvider := self class urlProvider ]! !

!GTUrlProvider methodsFor: 'initialize-release' stamp: 'TudorGirba 10/22/2014 07:49'!
urlProvider: anUrlObject
	urlProvider := anUrlObject asZnUrl ! !
GTPlayBook initialize!
